package xyz.wbsite.util;

import android.content.Context;
import android.content.Intent;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.BitmapFactory.Options;
import android.graphics.Matrix;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.DisplayMetrics;
import android.util.TypedValue;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Bitmap相关操作集合
 */
public final class BitmapUtil {

    /**
     * @param src        源Bitmap
     * @param destWidth  目标宽度
     * @param destHeigth 目标高度
     * @return Bitmap
     */
    public static Bitmap scaleBitmap(Bitmap src, int destWidth, int destHeigth) {
        if (src == null) return null;

        int width = src.getWidth();
        int height = src.getHeight();

        float scaleWidth = ((float) destWidth) / width;
        float scaleHeight = ((float) destHeigth) / height;

        Matrix matrix = new Matrix();
        matrix.postScale(scaleWidth, scaleHeight);
        return Bitmap.createBitmap(src, 0, 0, width, height, matrix, true);
    }

    /**
     * @param src   源Bitmap
     * @param scale 目标比例
     * @return Bitmap
     */
    public static Bitmap scaleBitmap(Bitmap src, float scale) {
        if (src == null) return null;

        int width = src.getWidth();
        int height = src.getHeight();

        Matrix matrix = new Matrix();
        matrix.postScale(scale, scale);
        return Bitmap.createBitmap(src, 0, 0, width, height, matrix, true);
    }

    /**
     * 从Asset中获取Drawable对象
     *
     * @param res      Resources
     * @param fileName 文件名
     * @return
     */
    public static Drawable getDrawableFromAssetFile(Resources res, String fileName) {
        InputStream is = null;
        Drawable icon = null;
        try {
            is = res.getAssets().open(fileName);
            TypedValue typedValue = new TypedValue();
            typedValue.density = TypedValue.DENSITY_DEFAULT;
            icon = Drawable.createFromResourceStream(res, typedValue, is, null);
        } catch (Exception e) {
        } finally {
            try {
                if (is != null) {
                    is.close();
                    is = null;
                }
            } catch (final IOException e) {
            }
        }
        return icon;
    }

    /**
     * 从Asset中获取位图对象
     *
     * @param res      Resources
     * @param fileName 文件名
     * @return
     */
    public static Bitmap getBitmapFromAssetFile(Resources res, String fileName) {
        InputStream is = null;
        Bitmap bmp = null;
        try {
            is = res.getAssets().open(fileName);

            DisplayMetrics metrics = res.getDisplayMetrics();
            Options opts = new Options();
            opts.inDensity = (int) (metrics.densityDpi / metrics.scaledDensity);
            bmp = BitmapFactory.decodeStream(is, null, opts);
            is.close();
            is = null;
        } catch (Exception e) {
        } finally {
            try {
                if (is != null) is.close();
            } catch (final IOException e) {
            }
        }
        return bmp;
    }

    /**
     * 从文件中获取Drawable对象
     *
     * @param res      Resources
     * @param fileName 文件名
     * @return
     */
    public static Drawable getDrawableFromFile(Resources res, String fileName) {
        FileInputStream fis = null;
        Drawable icon = null;
        try {
            fis = new FileInputStream(fileName);
            DisplayMetrics metrics = res.getDisplayMetrics();
            Options opts = new Options();
            opts.inDensity = (int) (metrics.densityDpi / metrics.scaledDensity);
            Bitmap bmp = BitmapFactory.decodeStream(fis, null, opts);

            fis.close();
            fis = null;
            if (bmp != null) {
                icon = new BitmapDrawable(bmp);
            }
        } catch (Exception e) {
        } finally {
            try {
                if (fis != null) fis.close();
            } catch (final IOException e) {
            }
        }
        return icon;
    }

    /**
     * 从文件中获取Bitmap对象
     *
     * @param res      Resources
     * @param fileName 文件名
     * @return
     */
    public static Bitmap getBitmapFromFile(Resources res, String fileName) {
        FileInputStream fis = null;
        Bitmap bmp = null;
        try {
            fis = new FileInputStream(fileName);
            DisplayMetrics metrics = res.getDisplayMetrics();
            Options opts = new Options();
            opts.inDensity = (int) (metrics.densityDpi / metrics.scaledDensity);
            bmp = BitmapFactory.decodeStream(fis, null, opts);
            fis.close();
            fis = null;
        } catch (Exception e) {
        } finally {
            try {
                if (fis != null) fis.close();
            } catch (final IOException e) {
            }
        }
        return bmp;
    }

    /**
     * @param bmp     保存bitmap到相册
     * @param picName 自定义的图片名
     */
    public static void saveBmp2Gallery(Context context, Bitmap bmp, String picName) {
        String fileName = null;
        //系统相册目录
        String galleryPath = Environment.getExternalStorageDirectory()
                + File.separator + Environment.DIRECTORY_DCIM
                + File.separator + "Camera" + File.separator;

        // 声明文件对象
        File file = null;
        // 声明输出流
        FileOutputStream outStream = null;

        try {
            // 如果有目标文件，直接获得文件对象，否则创建一个以filename为名称的文件
            file = new File(galleryPath, picName + ".jpg");

            // 获得文件相对路径
            fileName = file.toString();
            // 获得输出流，如果文件中有内容，追加内容
            outStream = new FileOutputStream(fileName);
            if (null != outStream) {
                bmp.compress(Bitmap.CompressFormat.JPEG, 100, outStream);
            }

        } catch (Exception e) {
            e.getStackTrace();
        } finally {
            try {
                if (outStream != null) {
                    outStream.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        //通知相册更新
        MediaStore.Images.Media.insertImage(context.getContentResolver(),
                bmp, fileName, null);
        Intent intent = new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
        Uri uri = Uri.fromFile(file);
        intent.setData(uri);
        context.sendBroadcast(intent);

        Toaster.showToast("图片保存成功");
    }

}
