package xyz.wbsite.util;


import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Environment;
import android.os.StatFs;
import android.util.Log;

import java.io.File;
import java.io.IOException;

import static android.os.Environment.MEDIA_MOUNTED;

public class StorageUtil {
    private static final String EXTERNAL_STORAGE_PERMISSION = "android.permission.WRITE_EXTERNAL_STORAGE";
    private static final String TAG = "StorageUtils";

    /**
     * 获取缓存目录
     * context 上下文
     */
    public static File getCacheDir(Context context) {
        return getCacheDir(context, true);
    }

    /**
     * 获取文件目录
     * context 上下文
     */
    public static File getFilesDir(Context context) {
        return getFilesDir(context, true);
    }

    /**
     * 获取缓存目录 当{preferExternal}为true时优先获取SD缓存目录，false或无权限或获取失败时则返回内部缓存目录
     * context 上下文
     * preferExternal 是否外部优先
     */
    public static File getCacheDir(Context context, boolean preferExternal) {
        File appCacheDir = null;
        if (preferExternal && MEDIA_MOUNTED.equals(Environment.getExternalStorageState()) && hasExternalStoragePermission(context)) {
            appCacheDir = context.getExternalCacheDir();
            if (!appCacheDir.exists()) {
                if (!appCacheDir.mkdirs()) {
                    Log.d(TAG, "Unable to create external cache directory");
                    return null;
                }
                try {
                    new File(appCacheDir, ".nomedia").createNewFile();
                } catch (IOException e) {
                    Log.d(TAG, "Can't create \".nomedia\" file in application external cache directory");
                }
            }
            return appCacheDir;
        }
        appCacheDir = context.getCacheDir();
        return appCacheDir;
    }

    /**
     * 获取文件目录 当{preferExternal}为true时优先获取SD文件目录，false或无权限或获取失败时则返回内部文件目录
     * context 上下文
     * preferExternal 是否外部优先
     */
    public static File getFilesDir(Context context, boolean preferExternal) {
        File appFilesDir = null;
        if (preferExternal && MEDIA_MOUNTED.equals(Environment.getExternalStorageState()) && hasExternalStoragePermission(context)) {
            appFilesDir = context.getExternalFilesDir("");
            if (!appFilesDir.exists()) {
                if (!appFilesDir.mkdirs()) {
                    return null;
                }
            }
            return appFilesDir;
        }
        appFilesDir = context.getFilesDir();
        return appFilesDir;
    }

    /**
     * 返回自定义缓存目录。在SD卡根目录上创建
     * 保证系统的健壮，万不得已的情况下返回系统缓存目录
     *
     * @param context Application context
     * @param dir     Cache directory path (e.g.: "AppCacheDir", "AppDir/cache/images")
     * @return Cache {@link File directory}
     */
    public static File getLocalDir(Context context, String dir) {
        File appCacheDir = null;
        if (MEDIA_MOUNTED.equals(Environment.getExternalStorageState()) && hasExternalStoragePermission(context)) {
            appCacheDir = new File(Environment.getExternalStorageDirectory(), dir);
        }
        if (appCacheDir == null || (!appCacheDir.exists() && !appCacheDir.mkdirs())) {
            appCacheDir = context.getCacheDir();
        }
        return appCacheDir;
    }

    private static boolean hasExternalStoragePermission(Context context) {
        int perm = context.checkCallingOrSelfPermission(EXTERNAL_STORAGE_PERMISSION);
        return perm == PackageManager.PERMISSION_GRANTED;
    }

    /**
     * @return SDCard是否可用
     */
    private static boolean isSDCardEnable() {
        return Environment.getExternalStorageState().equals(
                Environment.MEDIA_MOUNTED);

    }

    /**
     * @return 获取SD卡路径
     */
    private static String getSDCardPath() {
        return Environment.getExternalStorageDirectory().getAbsolutePath()
                + File.separator;
    }

    /**
     * @return SD卡的剩余容量 单位byte
     */
    private static long getSDCardAllSize() {
        if (isSDCardEnable()) {
            StatFs stat = new StatFs(getSDCardPath());
            // 获取空闲的数据块的数量
            long availableBlocks = (long) stat.getAvailableBlocks() - 4;
            // 获取单个数据块的大小（byte）
            long freeBlocks = stat.getAvailableBlocks();
            return freeBlocks * availableBlocks;
        }
        return 0;
    }
}